/***********************************************************************

Vorosketch User Distances Demo by Herman Haverkort, 25 May 2022

************************************************************************

Copyright 2022 Herman Johannes Haverkort

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This file is designed to be used with Vorosketch v0.14 BETA; it might 
work with later versions too. New versions may be available from 
http://herman.haverkort.net.


***********************************************************************/


#define MAKEUSERDISTANCE \
if (dname == "mydistance") distance = new MyDistance(); else \
if (dname == "quotient") distance = new QuotientDistance();


class MyDistance: public Distance
{
    public:
  virtual const string description() const
  {
    return "My Distance (default Euclidean)";
  }

  virtual bool supportsPolylines() const
  {
    return false;
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    return (p - s.points[0]).r();
  }
};


class QuotientDistance: public Distance
{
    public:
  virtual const string description() const
  {
    return "quotient distance (y/x)";
  }

  virtual bool supportsPolylines() const
  {
    return false;
  }

  virtual DOUBLE operator()(Point& p, const Site& s) const
  {
    if (p == s.points[0]) return 0;
    if (p.x() == s.points[0].x()) return INF;
    return abs(p.y() - s.points[0].y()) / abs(p.x() - s.points[0].x());
  }
};
